@@ -43,14 +43,11 @@ typedef struct ALechoState {
         ALuint delay;
     } Tap[2];
     ALuint Offset;
-    // The LR gains for the first tap. The second tap uses the reverse
-    ALfloat GainL;
-    ALfloat GainR;
+    /* The panning gains for the two taps */
+    ALfloat Gain[2][MAXCHANNELS];
 
     ALfloat FeedGain;
 
-    ALfloat Gain[MAXCHANNELS];
-
     FILTER iirFilter;
     ALfloat history[2];
 } ALechoState;
@@ -73,8 +70,8 @@ static ALboolean EchoDeviceUpdate(ALeffectState *effect, ALCdevice *Device)
 
     // Use the next power of 2 for the buffer length, so the tap offsets can be
     // wrapped using a mask instead of a modulo
-    maxlen  = (ALuint)(AL_ECHO_MAX_DELAY * Device->Frequency) + 1;
-    maxlen += (ALuint)(AL_ECHO_MAX_LRDELAY * Device->Frequency) + 1;
+    maxlen  = fastf2u(AL_ECHO_MAX_DELAY * Device->Frequency) + 1;
+    maxlen += fastf2u(AL_ECHO_MAX_LRDELAY * Device->Frequency) + 1;
     maxlen  = NextPowerOf2(maxlen);
 
     if(maxlen != state->BufferLength)
@@ -90,78 +87,87 @@ static ALboolean EchoDeviceUpdate(ALeffectState *effect, ALCdevice *Device)
     for(i = 0;i < state->BufferLength;i++)
         state->SampleBuffer[i] = 0.0f;
 
-    for(i = 0;i < MAXCHANNELS;i++)
-        state->Gain[i] = 0.0f;
-    for(i = 0;i < Device->NumChan;i++)
-    {
-        Channel chan = Device->Speaker2Chan[i];
-        state->Gain[chan] = 1.0f;
-    }
-
     return AL_TRUE;
 }
 
-static ALvoid EchoUpdate(ALeffectState *effect, ALCcontext *Context, const ALeffect *Effect)
+static ALvoid EchoUpdate(ALeffectState *effect, ALCdevice *Device, const ALeffectslot *Slot)
 {
     ALechoState *state = (ALechoState*)effect;
-    ALuint frequency = Context->Device->Frequency;
-    ALfloat lrpan, cw, a, g;
+    ALuint frequency = Device->Frequency;
+    ALfloat dirGain, ambientGain;
+    const ALfloat *ChannelGain;
+    ALfloat lrpan, cw, g, gain;
+    ALuint i, pos;
 
-    state->Tap[0].delay = (ALuint)(Effect->Echo.Delay * frequency) + 1;
-    state->Tap[1].delay = (ALuint)(Effect->Echo.LRDelay * frequency);
+    state->Tap[0].delay = fastf2u(Slot->effect.Echo.Delay * frequency) + 1;
+    state->Tap[1].delay = fastf2u(Slot->effect.Echo.LRDelay * frequency);
     state->Tap[1].delay += state->Tap[0].delay;
 
-    lrpan = Effect->Echo.Spread*0.5f + 0.5f;
-    state->GainL = aluSqrt(     lrpan);
-    state->GainR = aluSqrt(1.0f-lrpan);
+    lrpan = Slot->effect.Echo.Spread;
 
-    state->FeedGain = Effect->Echo.Feedback;
+    state->FeedGain = Slot->effect.Echo.Feedback;
 
-    cw = cos(2.0*M_PI * LOWPASSFREQCUTOFF / frequency);
-    g = 1.0f - Effect->Echo.Damping;
-    a = 0.0f;
-    if(g < 0.9999f) // 1-epsilon
-        a = (1 - g*cw - aluSqrt(2*g*(1-cw) - g*g*(1 - cw*cw))) / (1 - g);
-    state->iirFilter.coeff = a;
+    cw = aluCos(F_PI*2.0f * LOWPASSFREQREF / frequency);
+    g = 1.0f - Slot->effect.Echo.Damping;
+    state->iirFilter.coeff = lpCoeffCalc(g, cw);
+
+    gain = Slot->Gain;
+    for(i = 0;i < MAXCHANNELS;i++)
+    {
+        state->Gain[0][i] = 0.0f;
+        state->Gain[1][i] = 0.0f;
+    }
+
+    ambientGain = aluSqrt(1.0f/Device->NumChan);
+    dirGain = aluFabs(lrpan);
+
+    /* First tap panning */
+    pos = aluCart2LUTpos(0.0f, ((lrpan>0.0f)?-1.0f:1.0f));
+    ChannelGain = Device->PanningLUT[pos];
+
+    for(i = 0;i < Device->NumChan;i++)
+    {
+        enum Channel chan = Device->Speaker2Chan[i];
+        state->Gain[0][chan] = lerp(ambientGain, ChannelGain[chan], dirGain) * gain;
+    }
+
+    /* Second tap panning */
+    pos = aluCart2LUTpos(0.0f, ((lrpan>0.0f)?1.0f:-1.0f));
+    ChannelGain = Device->PanningLUT[pos];
+
+    for(i = 0;i < Device->NumChan;i++)
+    {
+        enum Channel chan = Device->Speaker2Chan[i];
+        state->Gain[1][chan] = lerp(ambientGain, ChannelGain[chan], dirGain) * gain;
+    }
 }
 
-static ALvoid EchoProcess(ALeffectState *effect, const ALeffectslot *Slot, ALuint SamplesToDo, const ALfloat *SamplesIn, ALfloat (*SamplesOut)[MAXCHANNELS])
+static ALvoid EchoProcess(ALeffectState *effect, ALuint SamplesToDo, const ALfloat *SamplesIn, ALfloat (*SamplesOut)[MAXCHANNELS])
 {
     ALechoState *state = (ALechoState*)effect;
     const ALuint mask = state->BufferLength-1;
     const ALuint tap1 = state->Tap[0].delay;
     const ALuint tap2 = state->Tap[1].delay;
     ALuint offset = state->Offset;
-    const ALfloat gain = Slot->Gain;
-    ALfloat samp[2], smp;
-    ALuint i;
+    ALfloat smp;
+    ALuint i, k;
 
     for(i = 0;i < SamplesToDo;i++,offset++)
     {
-        // Sample first tap
+        /* First tap */
         smp = state->SampleBuffer[(offset-tap1) & mask];
-        samp[0] = smp * state->GainL;
-        samp[1] = smp * state->GainR;
-        // Sample second tap. Reverse LR panning
+        for(k = 0;k < MAXCHANNELS;k++)
+            SamplesOut[i][k] += smp * state->Gain[0][k];
+
+        /* Second tap */
         smp = state->SampleBuffer[(offset-tap2) & mask];
-        samp[0] += smp * state->GainR;
-        samp[1] += smp * state->GainL;
+        for(k = 0;k < MAXCHANNELS;k++)
+            SamplesOut[i][k] += smp * state->Gain[1][k];
 
         // Apply damping and feedback gain to the second tap, and mix in the
         // new sample
         smp = lpFilter2P(&state->iirFilter, 0, smp+SamplesIn[i]);
         state->SampleBuffer[offset&mask] = smp * state->FeedGain;
-
-        // Apply slot gain
-        samp[0] *= gain;
-        samp[1] *= gain;
-
-        SamplesOut[i][FRONT_LEFT]  += state->Gain[FRONT_LEFT]  * samp[0];
-        SamplesOut[i][FRONT_RIGHT] += state->Gain[FRONT_RIGHT] * samp[1];
-        SamplesOut[i][SIDE_LEFT]   += state->Gain[SIDE_LEFT]   * samp[0];
-        SamplesOut[i][SIDE_RIGHT]  += state->Gain[SIDE_RIGHT]  * samp[1];
-        SamplesOut[i][BACK_LEFT]   += state->Gain[BACK_LEFT]   * samp[0];
-        SamplesOut[i][BACK_RIGHT]  += state->Gain[BACK_RIGHT]  * samp[1];
     }
     state->Offset = offset;
 }
@@ -185,8 +191,6 @@ ALeffectState *EchoCreate(void)
     state->Tap[0].delay = 0;
     state->Tap[1].delay = 0;
     state->Offset = 0;
-    state->GainL = 0.0f;
-    state->GainR = 0.0f;
 
     state->iirFilter.coeff = 0.0f;
     state->iirFilter.history[0] = 0.0f;
