@@ -29,157 +29,17 @@
 #include "alFilter.h"
 #include "alEffect.h"
 #include "alAuxEffectSlot.h"
-#include "alDatabuffer.h"
 #include "alSource.h"
 #include "alBuffer.h"
 #include "AL/al.h"
 #include "AL/alc.h"
 
-typedef struct ALfunction {
-    const ALchar *funcName;
-    ALvoid *address;
-} ALfunction;
-
 typedef struct ALenums {
     const ALchar *enumName;
     ALenum value;
 } ALenums;
 
 
-static const ALfunction function[] = {
-    { "alEnable",                   (ALvoid *) alEnable                  },
-    { "alDisable",                  (ALvoid *) alDisable                 },
-    { "alIsEnabled",                (ALvoid *) alIsEnabled               },
-    { "alGetString",                (ALvoid *) alGetString               },
-    { "alGetBooleanv",              (ALvoid *) alGetBooleanv             },
-    { "alGetIntegerv",              (ALvoid *) alGetIntegerv             },
-    { "alGetFloatv",                (ALvoid *) alGetFloatv               },
-    { "alGetDoublev",               (ALvoid *) alGetDoublev              },
-    { "alGetBoolean",               (ALvoid *) alGetBoolean              },
-    { "alGetInteger",               (ALvoid *) alGetInteger              },
-    { "alGetFloat",                 (ALvoid *) alGetFloat                },
-    { "alGetDouble",                (ALvoid *) alGetDouble               },
-    { "alGetError",                 (ALvoid *) alGetError                },
-    { "alIsExtensionPresent",       (ALvoid *) alIsExtensionPresent      },
-    { "alGetProcAddress",           (ALvoid *) alGetProcAddress          },
-    { "alGetEnumValue",             (ALvoid *) alGetEnumValue            },
-    { "alListenerf",                (ALvoid *) alListenerf               },
-    { "alListener3f",               (ALvoid *) alListener3f              },
-    { "alListenerfv",               (ALvoid *) alListenerfv              },
-    { "alListeneri",                (ALvoid *) alListeneri               },
-    { "alListener3i",               (ALvoid *) alListener3i              },
-    { "alListeneriv",               (ALvoid *) alListeneriv              },
-    { "alGetListenerf",             (ALvoid *) alGetListenerf            },
-    { "alGetListener3f",            (ALvoid *) alGetListener3f           },
-    { "alGetListenerfv",            (ALvoid *) alGetListenerfv           },
-    { "alGetListeneri",             (ALvoid *) alGetListeneri            },
-    { "alGetListener3i",            (ALvoid *) alGetListener3i           },
-    { "alGetListeneriv",            (ALvoid *) alGetListeneriv           },
-    { "alGenSources",               (ALvoid *) alGenSources              },
-    { "alDeleteSources",            (ALvoid *) alDeleteSources           },
-    { "alIsSource",                 (ALvoid *) alIsSource                },
-    { "alSourcef",                  (ALvoid *) alSourcef                 },
-    { "alSource3f",                 (ALvoid *) alSource3f                },
-    { "alSourcefv",                 (ALvoid *) alSourcefv                },
-    { "alSourcei",                  (ALvoid *) alSourcei                 },
-    { "alSource3i",                 (ALvoid *) alSource3i                },
-    { "alSourceiv",                 (ALvoid *) alSourceiv                },
-    { "alGetSourcef",               (ALvoid *) alGetSourcef              },
-    { "alGetSource3f",              (ALvoid *) alGetSource3f             },
-    { "alGetSourcefv",              (ALvoid *) alGetSourcefv             },
-    { "alGetSourcei",               (ALvoid *) alGetSourcei              },
-    { "alGetSource3i",              (ALvoid *) alGetSource3i             },
-    { "alGetSourceiv",              (ALvoid *) alGetSourceiv             },
-    { "alSourcePlayv",              (ALvoid *) alSourcePlayv             },
-    { "alSourceStopv",              (ALvoid *) alSourceStopv             },
-    { "alSourceRewindv",            (ALvoid *) alSourceRewindv           },
-    { "alSourcePausev",             (ALvoid *) alSourcePausev            },
-    { "alSourcePlay",               (ALvoid *) alSourcePlay              },
-    { "alSourceStop",               (ALvoid *) alSourceStop              },
-    { "alSourceRewind",             (ALvoid *) alSourceRewind            },
-    { "alSourcePause",              (ALvoid *) alSourcePause             },
-    { "alSourceQueueBuffers",       (ALvoid *) alSourceQueueBuffers      },
-    { "alSourceUnqueueBuffers",     (ALvoid *) alSourceUnqueueBuffers    },
-    { "alGenBuffers",               (ALvoid *) alGenBuffers              },
-    { "alDeleteBuffers",            (ALvoid *) alDeleteBuffers           },
-    { "alIsBuffer",                 (ALvoid *) alIsBuffer                },
-    { "alBufferData",               (ALvoid *) alBufferData              },
-    { "alBufferf",                  (ALvoid *) alBufferf                 },
-    { "alBuffer3f",                 (ALvoid *) alBuffer3f                },
-    { "alBufferfv",                 (ALvoid *) alBufferfv                },
-    { "alBufferi",                  (ALvoid *) alBufferi                 },
-    { "alBuffer3i",                 (ALvoid *) alBuffer3i                },
-    { "alBufferiv",                 (ALvoid *) alBufferiv                },
-    { "alGetBufferf",               (ALvoid *) alGetBufferf              },
-    { "alGetBuffer3f",              (ALvoid *) alGetBuffer3f             },
-    { "alGetBufferfv",              (ALvoid *) alGetBufferfv             },
-    { "alGetBufferi",               (ALvoid *) alGetBufferi              },
-    { "alGetBuffer3i",              (ALvoid *) alGetBuffer3i             },
-    { "alGetBufferiv",              (ALvoid *) alGetBufferiv             },
-    { "alDopplerFactor",            (ALvoid *) alDopplerFactor           },
-    { "alDopplerVelocity",          (ALvoid *) alDopplerVelocity         },
-    { "alSpeedOfSound",             (ALvoid *) alSpeedOfSound            },
-    { "alDistanceModel",            (ALvoid *) alDistanceModel           },
-
-    { "alGenFilters",               (ALvoid *) alGenFilters              },
-    { "alDeleteFilters",            (ALvoid *) alDeleteFilters           },
-    { "alIsFilter",                 (ALvoid *) alIsFilter                },
-    { "alFilteri",                  (ALvoid *) alFilteri                 },
-    { "alFilteriv",                 (ALvoid *) alFilteriv                },
-    { "alFilterf",                  (ALvoid *) alFilterf                 },
-    { "alFilterfv",                 (ALvoid *) alFilterfv                },
-    { "alGetFilteri",               (ALvoid *) alGetFilteri              },
-    { "alGetFilteriv",              (ALvoid *) alGetFilteriv             },
-    { "alGetFilterf",               (ALvoid *) alGetFilterf              },
-    { "alGetFilterfv",              (ALvoid *) alGetFilterfv             },
-
-    { "alGenEffects",               (ALvoid *) alGenEffects              },
-    { "alDeleteEffects",            (ALvoid *) alDeleteEffects           },
-    { "alIsEffect",                 (ALvoid *) alIsEffect                },
-    { "alEffecti",                  (ALvoid *) alEffecti                 },
-    { "alEffectiv",                 (ALvoid *) alEffectiv                },
-    { "alEffectf",                  (ALvoid *) alEffectf                 },
-    { "alEffectfv",                 (ALvoid *) alEffectfv                },
-    { "alGetEffecti",               (ALvoid *) alGetEffecti              },
-    { "alGetEffectiv",              (ALvoid *) alGetEffectiv             },
-    { "alGetEffectf",               (ALvoid *) alGetEffectf              },
-    { "alGetEffectfv",              (ALvoid *) alGetEffectfv             },
-
-    { "alGenAuxiliaryEffectSlots",  (ALvoid *) alGenAuxiliaryEffectSlots },
-    { "alDeleteAuxiliaryEffectSlots",(ALvoid *) alDeleteAuxiliaryEffectSlots},
-    { "alIsAuxiliaryEffectSlot",    (ALvoid *) alIsAuxiliaryEffectSlot   },
-    { "alAuxiliaryEffectSloti",     (ALvoid *) alAuxiliaryEffectSloti    },
-    { "alAuxiliaryEffectSlotiv",    (ALvoid *) alAuxiliaryEffectSlotiv   },
-    { "alAuxiliaryEffectSlotf",     (ALvoid *) alAuxiliaryEffectSlotf    },
-    { "alAuxiliaryEffectSlotfv",    (ALvoid *) alAuxiliaryEffectSlotfv   },
-    { "alGetAuxiliaryEffectSloti",  (ALvoid *) alGetAuxiliaryEffectSloti },
-    { "alGetAuxiliaryEffectSlotiv", (ALvoid *) alGetAuxiliaryEffectSlotiv},
-    { "alGetAuxiliaryEffectSlotf",  (ALvoid *) alGetAuxiliaryEffectSlotf },
-    { "alGetAuxiliaryEffectSlotfv", (ALvoid *) alGetAuxiliaryEffectSlotfv},
-
-    { "alBufferSubDataEXT",         (ALvoid *) alBufferSubDataEXT        },
-
-    { "alGenDatabuffersEXT",        (ALvoid *) alGenDatabuffersEXT       },
-    { "alDeleteDatabuffersEXT",     (ALvoid *) alDeleteDatabuffersEXT    },
-    { "alIsDatabufferEXT",          (ALvoid *) alIsDatabufferEXT         },
-    { "alDatabufferDataEXT",        (ALvoid *) alDatabufferDataEXT       },
-    { "alDatabufferSubDataEXT",     (ALvoid *) alDatabufferSubDataEXT    },
-    { "alGetDatabufferSubDataEXT",  (ALvoid *) alGetDatabufferSubDataEXT },
-    { "alDatabufferfEXT",           (ALvoid *) alDatabufferfEXT          },
-    { "alDatabufferfvEXT",          (ALvoid *) alDatabufferfvEXT         },
-    { "alDatabufferiEXT",           (ALvoid *) alDatabufferiEXT          },
-    { "alDatabufferivEXT",          (ALvoid *) alDatabufferivEXT         },
-    { "alGetDatabufferfEXT",        (ALvoid *) alGetDatabufferfEXT       },
-    { "alGetDatabufferfvEXT",       (ALvoid *) alGetDatabufferfvEXT      },
-    { "alGetDatabufferiEXT",        (ALvoid *) alGetDatabufferiEXT       },
-    { "alGetDatabufferivEXT",       (ALvoid *) alGetDatabufferivEXT      },
-    { "alSelectDatabufferEXT",      (ALvoid *) alSelectDatabufferEXT     },
-    { "alMapDatabufferEXT",         (ALvoid *) alMapDatabufferEXT        },
-    { "alUnmapDatabufferEXT",       (ALvoid *) alUnmapDatabufferEXT      },
-
-    { NULL,                         (ALvoid *) NULL                      }
-};
-
 static const ALenums enumeration[] = {
     // Types
     { "AL_INVALID",                           AL_INVALID                          },
@@ -207,12 +67,15 @@ static const ALenums enumeration[] = {
     { "AL_MAX_DISTANCE",                      AL_MAX_DISTANCE                     },
     { "AL_SEC_OFFSET",                        AL_SEC_OFFSET                       },
     { "AL_SAMPLE_OFFSET",                     AL_SAMPLE_OFFSET                    },
+    { "AL_SAMPLE_RW_OFFSETS_SOFT",            AL_SAMPLE_RW_OFFSETS_SOFT           },
     { "AL_BYTE_OFFSET",                       AL_BYTE_OFFSET                      },
+    { "AL_BYTE_RW_OFFSETS_SOFT",              AL_BYTE_RW_OFFSETS_SOFT             },
     { "AL_SOURCE_TYPE",                       AL_SOURCE_TYPE                      },
     { "AL_STATIC",                            AL_STATIC                           },
     { "AL_STREAMING",                         AL_STREAMING                        },
     { "AL_UNDETERMINED",                      AL_UNDETERMINED                     },
     { "AL_METERS_PER_UNIT",                   AL_METERS_PER_UNIT                  },
+    { "AL_DIRECT_CHANNELS_SOFT",              AL_DIRECT_CHANNELS_SOFT             },
 
     // Source EFX Properties
     { "AL_DIRECT_FILTER",                     AL_DIRECT_FILTER                    },
@@ -264,20 +127,70 @@ static const ALenums enumeration[] = {
     { "AL_FORMAT_REAR16",                     AL_FORMAT_REAR16                    },
     { "AL_FORMAT_REAR32",                     AL_FORMAT_REAR32                    },
     { "AL_FORMAT_MONO_MULAW",                 AL_FORMAT_MONO_MULAW                },
-    { "AL_FORMAT_MONO_MULAW_EXT",             AL_FORMAT_MONO_MULAW                },
+    { "AL_FORMAT_MONO_MULAW_EXT",             AL_FORMAT_MONO_MULAW_EXT            },
     { "AL_FORMAT_STEREO_MULAW",               AL_FORMAT_STEREO_MULAW              },
-    { "AL_FORMAT_STEREO_MULAW_EXT",           AL_FORMAT_STEREO_MULAW              },
+    { "AL_FORMAT_STEREO_MULAW_EXT",           AL_FORMAT_STEREO_MULAW_EXT          },
     { "AL_FORMAT_QUAD_MULAW",                 AL_FORMAT_QUAD_MULAW                },
     { "AL_FORMAT_51CHN_MULAW",                AL_FORMAT_51CHN_MULAW               },
     { "AL_FORMAT_61CHN_MULAW",                AL_FORMAT_61CHN_MULAW               },
     { "AL_FORMAT_71CHN_MULAW",                AL_FORMAT_71CHN_MULAW               },
     { "AL_FORMAT_REAR_MULAW",                 AL_FORMAT_REAR_MULAW                },
+    { "AL_FORMAT_MONO_ALAW_EXT",              AL_FORMAT_MONO_ALAW_EXT             },
+    { "AL_FORMAT_STEREO_ALAW_EXT",            AL_FORMAT_STEREO_ALAW_EXT           },
+
+    // Internal Buffer Formats
+    { "AL_MONO8_SOFT",                        AL_MONO8_SOFT                       },
+    { "AL_MONO16_SOFT",                       AL_MONO16_SOFT                      },
+    { "AL_MONO32F_SOFT",                      AL_MONO32F_SOFT                     },
+    { "AL_STEREO8_SOFT",                      AL_STEREO8_SOFT                     },
+    { "AL_STEREO16_SOFT",                     AL_STEREO16_SOFT                    },
+    { "AL_STEREO32F_SOFT",                    AL_STEREO32F_SOFT                   },
+    { "AL_QUAD8_SOFT",                        AL_QUAD8_SOFT                       },
+    { "AL_QUAD16_SOFT",                       AL_QUAD16_SOFT                      },
+    { "AL_QUAD32F_SOFT",                      AL_QUAD32F_SOFT                     },
+    { "AL_REAR8_SOFT",                        AL_REAR8_SOFT                       },
+    { "AL_REAR16_SOFT",                       AL_REAR16_SOFT                      },
+    { "AL_REAR32F_SOFT",                      AL_REAR32F_SOFT                     },
+    { "AL_5POINT1_8_SOFT",                    AL_5POINT1_8_SOFT                   },
+    { "AL_5POINT1_16_SOFT",                   AL_5POINT1_16_SOFT                  },
+    { "AL_5POINT1_32F_SOFT",                  AL_5POINT1_32F_SOFT                 },
+    { "AL_6POINT1_8_SOFT",                    AL_6POINT1_8_SOFT                   },
+    { "AL_6POINT1_16_SOFT",                   AL_6POINT1_16_SOFT                  },
+    { "AL_6POINT1_32F_SOFT",                  AL_6POINT1_32F_SOFT                 },
+    { "AL_7POINT1_8_SOFT",                    AL_7POINT1_8_SOFT                   },
+    { "AL_7POINT1_16_SOFT",                   AL_7POINT1_16_SOFT                  },
+    { "AL_7POINT1_32F_SOFT",                  AL_7POINT1_32F_SOFT                 },
+
+    // Buffer Channel Configurations
+    { "AL_MONO_SOFT",                         AL_MONO_SOFT                        },
+    { "AL_STEREO_SOFT",                       AL_STEREO_SOFT                      },
+    { "AL_QUAD_SOFT",                         AL_QUAD_SOFT                        },
+    { "AL_REAR_SOFT",                         AL_REAR_SOFT                        },
+    { "AL_5POINT1_SOFT",                      AL_5POINT1_SOFT                     },
+    { "AL_6POINT1_SOFT",                      AL_6POINT1_SOFT                     },
+    { "AL_7POINT1_SOFT",                      AL_7POINT1_SOFT                     },
+
+    // Buffer Sample Types
+    { "AL_BYTE_SOFT",                         AL_BYTE_SOFT                        },
+    { "AL_UNSIGNED_BYTE_SOFT",                AL_UNSIGNED_BYTE_SOFT               },
+    { "AL_SHORT_SOFT",                        AL_SHORT_SOFT                       },
+    { "AL_UNSIGNED_SHORT_SOFT",               AL_UNSIGNED_SHORT_SOFT              },
+    { "AL_INT_SOFT",                          AL_INT_SOFT                         },
+    { "AL_UNSIGNED_INT_SOFT",                 AL_UNSIGNED_INT_SOFT                },
+    { "AL_FLOAT_SOFT",                        AL_FLOAT_SOFT                       },
+    { "AL_DOUBLE_SOFT",                       AL_DOUBLE_SOFT                      },
+    { "AL_BYTE3_SOFT",                        AL_BYTE3_SOFT                       },
+    { "AL_UNSIGNED_BYTE3_SOFT",               AL_UNSIGNED_BYTE3_SOFT              },
 
     // Buffer attributes
     { "AL_FREQUENCY",                         AL_FREQUENCY                        },
     { "AL_BITS",                              AL_BITS                             },
     { "AL_CHANNELS",                          AL_CHANNELS                         },
     { "AL_SIZE",                              AL_SIZE                             },
+    { "AL_INTERNAL_FORMAT_SOFT",              AL_INTERNAL_FORMAT_SOFT             },
+    { "AL_BYTE_LENGTH_SOFT",                  AL_BYTE_LENGTH_SOFT                 },
+    { "AL_SAMPLE_LENGTH_SOFT",                AL_SAMPLE_LENGTH_SOFT               },
+    { "AL_SEC_LENGTH_SOFT",                   AL_SEC_LENGTH_SOFT                  },
 
     // Buffer States (not supported yet)
     { "AL_UNUSED",                            AL_UNUSED                           },
@@ -303,6 +216,8 @@ static const ALenums enumeration[] = {
     { "AL_DOPPLER_VELOCITY",                  AL_DOPPLER_VELOCITY                 },
     { "AL_DISTANCE_MODEL",                    AL_DISTANCE_MODEL                   },
     { "AL_SPEED_OF_SOUND",                    AL_SPEED_OF_SOUND                   },
+    { "AL_SOURCE_DISTANCE_MODEL",             AL_SOURCE_DISTANCE_MODEL            },
+    { "AL_DEFERRED_UPDATES_SOFT",             AL_DEFERRED_UPDATES_SOFT            },
 
     // Distance Models
     { "AL_INVERSE_DISTANCE",                  AL_INVERSE_DISTANCE                 },
@@ -340,11 +255,15 @@ static const ALenums enumeration[] = {
     { "AL_EFFECT_FREQUENCY_SHIFTER",          AL_EFFECT_FREQUENCY_SHIFTER         },
     { "AL_EFFECT_VOCAL_MORPHER",              AL_EFFECT_VOCAL_MORPHER             },
     { "AL_EFFECT_PITCH_SHIFTER",              AL_EFFECT_PITCH_SHIFTER             },
+#endif
     { "AL_EFFECT_RING_MODULATOR",             AL_EFFECT_RING_MODULATOR            },
+#if 0
     { "AL_EFFECT_AUTOWAH",                    AL_EFFECT_AUTOWAH                   },
     { "AL_EFFECT_COMPRESSOR",                 AL_EFFECT_COMPRESSOR                },
     { "AL_EFFECT_EQUALIZER",                  AL_EFFECT_EQUALIZER                 },
 #endif
+    { "AL_EFFECT_DEDICATED_LOW_FREQUENCY_EFFECT",AL_EFFECT_DEDICATED_LOW_FREQUENCY_EFFECT},
+    { "AL_EFFECT_DEDICATED_DIALOGUE",         AL_EFFECT_DEDICATED_DIALOGUE        },
 
     // Reverb params
     { "AL_REVERB_DENSITY",                    AL_REVERB_DENSITY                   },
@@ -393,68 +312,88 @@ static const ALenums enumeration[] = {
     { "AL_ECHO_FEEDBACK",                     AL_ECHO_FEEDBACK                    },
     { "AL_ECHO_SPREAD",                       AL_ECHO_SPREAD                      },
 
+    // Ring Modulator params
+    { "AL_RING_MODULATOR_FREQUENCY",          AL_RING_MODULATOR_FREQUENCY         },
+    { "AL_RING_MODULATOR_HIGHPASS_CUTOFF",    AL_RING_MODULATOR_HIGHPASS_CUTOFF   },
+    { "AL_RING_MODULATOR_WAVEFORM",           AL_RING_MODULATOR_WAVEFORM          },
+
+    // Dedicated Dialogue/LFE params
+    { "AL_DEDICATED_GAIN",                    AL_DEDICATED_GAIN                   },
+
 
     // Default
     { NULL,                                   (ALenum)0                           }
 };
 
 
+const struct EffectList EffectList[] = {
+    { "eaxreverb", EAXREVERB, "AL_EFFECT_EAXREVERB",      AL_EFFECT_EAXREVERB },
+    { "reverb",    REVERB,    "AL_EFFECT_REVERB",         AL_EFFECT_REVERB },
+    { "echo",      ECHO,      "AL_EFFECT_ECHO",           AL_EFFECT_ECHO },
+    { "modulator", MODULATOR, "AL_EFFECT_RING_MODULATOR", AL_EFFECT_RING_MODULATOR },
+    { "dedicated", DEDICATED, "AL_EFFECT_DEDICATED_LOW_FREQUENCY_EFFECT", AL_EFFECT_DEDICATED_LOW_FREQUENCY_EFFECT },
+    { "dedicated", DEDICATED, "AL_EFFECT_DEDICATED_DIALOGUE", AL_EFFECT_DEDICATED_DIALOGUE },
+    { NULL, 0, NULL, (ALenum)0 }
+};
+
 
 AL_API ALboolean AL_APIENTRY alIsExtensionPresent(const ALchar *extName)
 {
     ALboolean bIsSupported = AL_FALSE;
-    ALCcontext *pContext;
+    ALCcontext *Context;
     const char *ptr;
     size_t len;
 
-    pContext = GetContextSuspended();
-    if(!pContext) return AL_FALSE;
+    Context = GetContextRef();
+    if(!Context) return AL_FALSE;
 
     if(!extName)
+        alSetError(Context, AL_INVALID_VALUE);
+    else
     {
-        alSetError(pContext, AL_INVALID_VALUE);
-        ProcessContext(pContext);
-        return AL_FALSE;
-    }
-
-    len = strlen(extName);
-    ptr = pContext->ExtensionList;
-    while(ptr && *ptr)
-    {
-        if(strncasecmp(ptr, extName, len) == 0 &&
-           (ptr[len] == '\0' || isspace(ptr[len])))
+        len = strlen(extName);
+        ptr = Context->ExtensionList;
+        while(ptr && *ptr)
         {
-            bIsSupported = AL_TRUE;
-            break;
-        }
-        if((ptr=strchr(ptr, ' ')) != NULL)
-        {
-            do {
-                ++ptr;
-            } while(isspace(*ptr));
+            if(strncasecmp(ptr, extName, len) == 0 &&
+               (ptr[len] == '\0' || isspace(ptr[len])))
+            {
+                bIsSupported = AL_TRUE;
+                break;
+            }
+            if((ptr=strchr(ptr, ' ')) != NULL)
+            {
+                do {
+                    ++ptr;
+                } while(isspace(*ptr));
+            }
         }
     }
 
-    ProcessContext(pContext);
-
+    ALCcontext_DecRef(Context);
     return bIsSupported;
 }
 
 
 AL_API ALvoid* AL_APIENTRY alGetProcAddress(const ALchar *funcName)
 {
-    ALsizei i = 0;
-
-    while(function[i].funcName && strcmp(function[i].funcName, funcName) != 0)
-        i++;
-
-    return function[i].address;
+    if(!funcName)
+        return NULL;
+    return alcGetProcAddress(NULL, funcName);
 }
 
 AL_API ALenum AL_APIENTRY alGetEnumValue(const ALchar *enumName)
 {
-    ALsizei i = 0;
+    ALsizei i;
 
+    for(i = 0;EffectList[i].ename;i++)
+    {
+        if(DisabledEffects[EffectList[i].type] &&
+           strcmp(EffectList[i].ename, enumName) == 0)
+            return (ALenum)0;
+    }
+
+    i = 0;
     while(enumeration[i].enumName &&
           strcmp(enumeration[i].enumName, enumName) != 0)
         i++;
